﻿using System;
using System.Collections.Generic;
using System.Text;
using nicoAE.Plugin.Interface;
using nicoAE.Plugin.Utils;
using nicoAE.Plugin.Controls;
using nicoAE.Plugin.Property;
using nicoAE.Plugin.Threading;
using System.Drawing;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;

namespace SamplePlugin
{
    //基本項目はすべて省略。詳しくはBrightnessクラスを参照のこと
    public class MultiThreadSample : IEffect
    {
        IHost ho = null;

        #region IEffect メンバ

        public string PluginName
        {
            get
            {
                return "サンプル(ぼかし)";
            }
        }

        public string Category
        {
            get
            {
                return "ブラー";
            }
        }

        public bool PresetProperty
        {
            get
            {
                return false;
            }
        }

        public MediaType CorrespondingType
        {
            get
            {
                return MediaType.Image | MediaType.Video;
            }
        }

        public void Initialize(IHost host, MediaInfo info)
        {
            //マルチスレッドを使用するかどうか判断するためにIHostを取得します
            ho = host;
        }

        public IPropertyEditControl[] GetControls()
        {
            return new IPropertyEditControl[] { new Int32EditControl("範囲") };
        }

        public IEffectProperty[] GetDefaultProperty()
        {
            return new IEffectProperty[] { new Int32Property("範囲", 1, 1000, 0) };
        }

        public KeyFrame[] GetPresetProperty(int index, string propertyName)
        {
            throw new NotImplementedException();
        }

        public Bitmap ProcessingImage(Bitmap image, IEffectProperty[] property)
        {
            //プロパティを取得します
            int zone = ((Int32Property)property[0]).TypeSafeValue;
            //範囲が0の場合は何もしない
            if (zone > 0)
            {
                //初期処理
                int width = image.Width;
                int height = image.Height;
                byte[] data = new byte[width * height * 4];
                byte[] temp = new byte[data.Length];
                BitmapData bd = image.LockBits(new Rectangle(new Point(), image.Size), ImageLockMode.ReadWrite, PixelFormat.Format32bppArgb);
                Marshal.Copy(bd.Scan0, data, 0, data.Length);
                //マルチスレッドを使用するかどうか。この項目はユーザーによって決定されます
                if (!ho.MultiThread)
                {
                    //高速化のために縦と横に分割して処理します
                    //まずは横から
                    for (int w = 0; w < width; w++)
                    {
                        int count = 0;
                        int a = 0;
                        int r = 0;
                        int g = 0;
                        int b = 0;
                        int position = 0;
                        for (int h = -zone; h < height; h++)
                        {
                            int t = h - zone;
                            if (t > -1)
                            {
                                position = (t * width + w) * 4;
                                byte ta = data[position + 3];
                                b -= data[position] * ta;
                                g -= data[position + 1] * ta;
                                r -= data[position + 2] * ta;
                                a -= ta;
                                count--;
                            }
                            t = h + zone;
                            if (t < height)
                            {
                                position = (t * width + w) * 4;
                                byte ta = data[position + 3];
                                b += data[position] * ta;
                                g += data[position + 1] * ta;
                                r += data[position + 2] * ta;
                                a += ta;
                                count++;
                            }
                            if (h > -1 && a > 0)
                            {
                                position = (h * width + w) * 4;
                                temp[position] = (byte)((double)b / a);
                                temp[position + 1] = (byte)((double)g / a);
                                temp[position + 2] = (byte)((double)r / a);
                                temp[position + 3] = (byte)(a / count);
                            }
                        }
                    }
                    //次に縦
                    for (int h = 0; h < height; h++)
                    {
                        int count = 0;
                        int a = 0;
                        int r = 0;
                        int g = 0;
                        int b = 0;
                        int position = 0;
                        for (int w = -zone; w < width; w++)
                        {
                            int l = w - zone;
                            if (l > -1)
                            {
                                position = (h * width + l) * 4;
                                byte ta = temp[position + 3];
                                b -= temp[position] * ta;
                                g -= temp[position + 1] * ta;
                                r -= temp[position + 2] * ta;
                                a -= ta;
                                count--;
                            }
                            l = w + zone;
                            if (l < width)
                            {
                                position = (h * width + l) * 4;
                                byte ta = temp[position + 3];
                                b += temp[position] * ta;
                                g += temp[position + 1] * ta;
                                r += temp[position + 2] * ta;
                                a += ta;
                                count++;
                            }
                            if (w > -1 && a > 0)
                            {
                                position = (h * width + w) * 4;
                                data[position] = (byte)((double)b / a);
                                data[position + 1] = (byte)((double)g / a);
                                data[position + 2] = (byte)((double)r / a);
                                data[position + 3] = (byte)(a / count);
                            }
                        }
                    }
                }
                else
                {
                    //処理方法は上と一緒。　上の処理の最上段のforをParallel.Forに
                    //置き換えてfor内部を引数がint1個の匿名メソッドに仕立てればOK
                    //ただし、指定する処理はちゃんとスレッドセーフにすること。
                    //スレッドセーフにしないとどうなるかは以下の目印に沿ってコメント
                    //にしたりコメント解除するとかわかります
                    /* スレッドセーフにしない場合はここをコメント解除
                     * int count = 0;
                     * int a = 0;
                     * int r = 0;
                     * int g = 0;
                     * int b = 0;
                     * int position = 0;
                     */
                    //for(int w = 0; w < width; w++)
                    Parallel.For(0, width, delegate(int w)
                    {
                        /* スレッドセーフにしない場合はここをコメント解除
                         * a = r = g = b = count = position = 0;
                         */
                        int count = 0;
                        int a = 0;
                        int r = 0;
                        int g = 0;
                        int b = 0;
                        int position = 0;
                        /*そしてここまでコメントアウト*/
                        for (int h = -zone; h < height; h++)
                        {
                            int t = h - zone;
                            if (t > -1)
                            {
                                position = (t * width + w) * 4;
                                byte ta = data[position + 3];
                                b -= data[position] * ta;
                                g -= data[position + 1] * ta;
                                r -= data[position + 2] * ta;
                                a -= ta;
                                count--;
                            }
                            t = h + zone;
                            if (t < height)
                            {
                                position = (t * width + w) * 4;
                                byte ta = data[position + 3];
                                b += data[position] * ta;
                                g += data[position + 1] * ta;
                                r += data[position + 2] * ta;
                                a += ta;
                                count++;
                            }
                            if (h > -1 && a > 0)
                            {
                                position = (h * width + w) * 4;
                                temp[position] = (byte)((double)b / a);
                                temp[position + 1] = (byte)((double)g / a);
                                temp[position + 2] = (byte)((double)r / a);
                                temp[position + 3] = (byte)(a / count);
                            }
                        }
                    });
                    //縦も同様
                    //for(int h = 0; h < height; h++)
                    Parallel.For(0, height, delegate(int h)
                    {
                        /* スレッドセーフにしない場合はここをコメント解除
                         * a = r = g = b = count = position = 0;
                         */
                        int count = 0;
                        int a = 0;
                        int r = 0;
                        int g = 0;
                        int b = 0;
                        int position = 0;
                        /*そしてここまでコメントアウト*/
                        for (int w = -zone; w < width; w++)
                        {
                            int l = w - zone;
                            if (l > -1)
                            {
                                position = (h * width + l) * 4;
                                byte ta = temp[position + 3];
                                b -= temp[position] * ta;
                                g -= temp[position + 1] * ta;
                                r -= temp[position + 2] * ta;
                                a -= ta;
                                count--;
                            }
                            l = w + zone;
                            if (l < width)
                            {
                                position = (h * width + l) * 4;
                                byte ta = temp[position + 3];
                                b += temp[position] * ta;
                                g += temp[position + 1] * ta;
                                r += temp[position + 2] * ta;
                                a += ta;
                                count++;
                            }
                            if (w > -1 && a > 0)
                            {
                                position = (h * width + w) * 4;
                                data[position] = (byte)((double)b / a);
                                data[position + 1] = (byte)((double)g / a);
                                data[position + 2] = (byte)((double)r / a);
                                data[position + 3] = (byte)(a / count);
                            }
                        }
                    });
                }
                //結果をimageに書き戻す
                Marshal.Copy(data, 0, bd.Scan0, temp.Length);
                image.UnlockBits(bd);
            }
            return image;
        }

        public byte[] ProcessingAudio(byte[] audio, IEffectProperty[] property)
        {
            throw new NotImplementedException();
        }

        #endregion

        #region IDisposable メンバ

        public void Dispose() { }

        #endregion
    }
}
