﻿using System;
using System.Collections.Generic;
using System.Text;
using nicoAE.Plugin.Interface;

namespace SamplePlugin
{
    //この属性は必ず付けて下さい。
    [Serializable]
    /// <summary>
    /// 倍精度浮動小数点の値を保持するプロパティです。
    /// </summary>
    public class DoubleProperty : IEffectProperty
    {
        /// <summary>
        /// このクラスのコンストラクタ。
        /// </summary>
        /// <param name="name">プロパティの名前。</param>
        /// <param name="defProp">プロパティの初期値。</param>
        /// <param name="max">プロパティに設定できる最大値。</param>
        /// <param name="min">プロパティに設定できる最小値。</param>
        /// <param name="disit">設定できる桁数。</param>
        public DoubleProperty(string name, double defProp, double max, double min, int disit)
        {
            n = name;
            val = defProp;
            maximum = max;
            minimum = min;
            dig = disit;
        }

        string n = "";
        double val = 0;
        double maximum = 0;
        double minimum = 0;
        int dig = 0;

        #region IEffectProperty メンバ

        /// <summary>
        /// プロパティの名前を取得します。
        /// </summary>
        public string PropertyName
        {
            get
            {
                return n;
            }
        }

        /// <summary>
        /// プロパティを取得します。
        /// </summary>
        public object Value
        {
            //このプロパティはシステムで使用されますが、速度を出すためには専用のプロパティを作成した下さい。
            get
            {
                return val;
            }
            set
            {
                val = Math.Round((double)value, dig, MidpointRounding.ToEven);
            }
        }

        #endregion

        /// <summary>
        /// プロパティをDouble型で取得、または設定します。
        /// </summary>
        public double TypeSafeValue
        {
            get
            {
                return val;
            }
            set
            {
                val = Math.Round(value, dig, MidpointRounding.ToEven);
            }
        }

        /// <summary>
        /// このプロパティに設定できる最大値を取得します。
        /// </summary>
        public double Maximum
        {
            get
            {
                return maximum;
            }
        }

        /// <summary>
        /// このプロパティに設定できる最小値を取得します。
        /// </summary>
        public double Minimum
        {
            get
            {
                return minimum;
            }
        }

        /// <summary>
        /// このプロパティに設定できる桁数を取得します。
        /// </summary>
        public int Digit
        {
            get
            {
                return dig;
            }
        }
    }
}
